-- Un commentaire simple commence par deux tirets.
-- et court jusqu'à la fin de la ligne.

--[[ Il est aussi possible d'écrire des commentaires
  sur plusieurs lignes avec cette syntaxe.

  Dans ce qui suit, le signe --> sera utilisé pour indiquer le
  texte qui s'affiche sur la console. Comme il commence par deux
  tirets, il est vu comme un commentaire simple par Lua. ]]

-- Affecte à la variable globale "a" la chaîne "Hello World":
a = "Hello World"
print(a) --> Hello World
a = 3.14 -- Réaffecte à la même variable globale un nombre
print(a) --> 3.14
-- Les opérations mathématiques usuelles peuvent être utilisées:
b = 1 + a * 2
print(b) --> 7.28
-- Une variable qui n'a pas reçu de valeur vaut nil :
print(c) --> nil

-- Plusieurs syntaxes sont possibles pour entrer des nombres :
print(10, 10.00, 1e1, 0xA, 0x1.4p+3) --> 10  10  10  10  10
-- Il en va de même pour les chaînes de caractères :
print("Abc", 'Abc', [[Abc]], '\65\098\x63') --> Abc Abc Abc Abc

-- La fonction standard type retourne le type d'une valeur Lua.
-- Il existe 8 types de données, que nous verrons plus tard :
print(type(3.14), type(1), type(0x1234))--> number number number
print(type("Hello World"), type(_VERSION)) --> string  string
print(type(nil), type(c)) --> nil  nil
print(type(false), type(true)) --> boolean  boolean
print(type({}), type(io)) --> table  table
print(type(print), type(type)) --> function  function
print(type(io.stderr)) --> userdata
print(type(coroutine.create(print))) --> thread

-- Une variable locale se définit avec le mot-clé local.
-- Sa visibilité s'étend à tout le bloc dans laquelle elle
-- est définie, depuis la ligne qui suit "local".
print(a) --> 3.14
do
  a = "globale"
  print(a) --> globale
  local a
  print(a) --> nil
  a = "locale"
  print(a) --> locale
  do
    local a = a .. 2
  print(a) --> locale2
  end
  print(a) --> locale
end
print(a) --> globale

-- Il est possible d'affecter plusieurs variables en même temps:
a, b = 1, 2
print(a,b) --> 1  2
a, b = b, a -- Permutation sans variable temporaire !
print(a,b) --> 2  1
-- Une fonction peut retourner plusieurs valeurs en Lua :
a, b = select(2, 3, 4, 5, 6)
print(a, b) --> 4  5

-- Une variable peut recevoir des valeurs de n'importe quel type
-- au cours de l'exécution du script. Y compris des fonctions!
a = (2^5 - 5)%10 -- ^ = mise à la puissance, % = modulo
print(type(a), a) --> number  7
a = a .. " nains" -- .. est l'opérateur de concaténation
print(type(a), a) --> string  7 nains
a = print  -- Car print est aussi une simple variable globale !
print(type(a), a) --> function  function: 0x12345678
-- (Le résultat précédent variera, car il affiche l'adresse en
-- mémoire de print). Soyons encore un peu plus tordus :
b = type
print = "Hello"
a(b(a), print) --> function  Hello

