-- Dclaration avance de fonctions locales
local single_page, read_file, urldecode, read_directory

-- Boucle principale du programme
local function main_server(root, port)
  local socket = require 'socket'
  root = root or "/var/www"
  local server = socket.bind("0.0.0.0", port or 8080)
  while true do
    local client = server:accept()
    local res, err = xpcall(single_page, 
      debug.traceback, client, root)
    if not res then
      -- Si une erreur s'est produite, renvoyer code HTTP 500
      client:send"HTTP/1.0 500 Internal Server Error\r\n\r\n"
      client:send("OUPS! Il doit y avoir un bogue :\r\n"..err)
    end
    client:close()
  end
end

-- Fonction appele pour chaque requte  servir
function single_page(client, root)
  -- Lit et interprte la 1re ligne de la requte
  local line = assert(client:receive())
  local verb, path, proto = 
    assert(line:match("([A-Z]+)%s+(%S+)%s+(HTTP.+)"))
  -- Dcode les lignes d'en-tte de la requte
  local request = {}
  repeat
    line = assert(client:receive())
    local param, value = line:match("(%S-)%s*:%s*(.*)")
    if param then
      request[param] = value
    end
  until line == ""
  -- Dcode les variables de formulaire passes via l'URL
  local filepath, vars = path:match("(.*)%?(.*)")
  if vars then
    path = filepath
    for k,v in vars:gmatch("([^&=]+)=([^&]+)") do
      request[urldecode(k)] = urldecode(v)
    end
  end
  -- Normalise le chemin du fichier, ajoutant v. "/index.html"
  path = root .. urldecode(path)
  path = path:gsub("/$", "/index.html")
  -- Lit le fichier demand et l'envoie avec le code HTTP 200
  local data = read_file(path)
  client:send(("HTTP/1.0 200 OK\r\nContent-Length: %d\r\n\z
    Server: Lua\r\n\r\n") : format(#data))
  client:send(data)
end

-- Dcode caractres spciaux dans l'URL (+ et %XX)
function urldecode(str)
  return str:gsub("%+", " "):gsub("%%(%w%w)", function(c)
    return string.char(tonumber(c, 16))
  end)
end

-- Lit un fichier et retourne son contenu dans une chane
function read_file(path, request)
  local f = assert(io.open(path, 'rb'))
  local content = f:read("*a")
  f:close()
  -- Cration de l'environnement pour les tags dynamiques
  local exports = {'assert', 'bit32', 'error', 'ipairs', 
    'math', 'next', 'pairs', 'select', 'string', 'table',
    'tonumber', 'tostring', 'type', 'unpack'}
  local env = {os={date=os.date, time=os.time, 
    difftime=os.difftime}, request=request}
  for _,n in ipairs(exports) do
    env[n] = _ENV[n]
  end
  function env.printf(...) 
    env.print(string.format(...)) 
  end
  -- Recherche les tags <?lua ?> et excute leur contenu
  content = content:gsub("()<%?[Ll][Uu][Aa](.-)%?>", 
    function(pos, chunk)
      local printed = ""
      env.print = function(...)
        for _,v in ipairs{...} do
          printed = printed .. tostring(v) .. " "
        end
      end
      local src = string.format("@%s[%d]", path, pos+5)
      local fct = load("return "..chunk, src, "t", env) or
        assert(load(chunk, src, "t", env))
      local result = fct() or ""
      return printed .. result
    end)
  return content
end

-- Lance le serveur avec les arguments passs au script
main_server(...)
