-- Dclaration des fonctions locales
local main_server, single_page
local read_file, urldecode, read_directory

-- Boucle principale du programme
function main_server(root, port)
  local socket = require 'socket'
  root = root or "/var/www"
  local server = socket.bind("0.0.0.0", port or 8080)
  server:settimeout(0)
  local sock_state = {} 
  while true do
    local sock_wait = {{server},{}} 
    local sock_ready, err = {} 
    for k, v in pairs(sock_state) do 
      table.insert(sock_wait[v[2]], k)
    end
    sock_ready[1], sock_ready[2], err = socket.select(
      sock_wait[1], sock_wait[2]) 
    for i = 1, 2 do
      for _, sock in ipairs(sock_ready[i]) do 
        if sock == server then 
          local client = server:accept()
          client:settimeout(0) 
          sock_state[client] = {coroutine.create(single_page),
            1, {client, root}} 
        else
          local state = sock_state[sock]
          local ok,res = coroutine.resume(state[1],
            table.unpack(state[3])) 
          if not res then
            sock:close() 
            sock_state[sock] = nil
          elseif ok then
            sock_state[sock][2] = res 
          else
            local err = res.."\n"..debug.traceback(
              sock_state[sock][1]) 
            sock:send"HTTP/1.0 500 Internal Server Error\r\n\r\n"
            sock:send("OUPS! Il doit y avoir un bogue :\r\n"..err)
            sock:close()
            sock_state[sock] = nil
          end
        end
      end
    end
  end
end

-- Yield jusqu' recevoir une ligne de donnes du client
local function receiveLine(client)
  local data = ""
  while true do
    coroutine.yield(1)
    local ok, err, partial = client:receive()
    if ok then return data..ok end
    if err == "timeout" then
      data = data..partial
    else
      error(err)
    end
  end
end

-- Yield jusqu' envoyer un bloc de donnes au client
local function send(client, data)
  while true do
    local pos = 0
    coroutine.yield(2)
    local ok, err, partial = client:send(data, pos+1)
    if ok then return end
    if err == "timeout" then
      pos = partial
    else
      error(err)
    end
  end
end
-- Fonction appele pour chaque requte  servir
function single_page(client, root)
  -- Lit et interprte la 1re ligne de la requte
  local line = receiveLine(client)
  local verb, path, proto =
    assert(line:match("([A-Z]+)%s+(%S+)%s+(HTTP.+)"))
  -- Dcode les lignes d'en-tte de la requte
  local request = {}
  repeat
    line = receiveLine(client)
    local param, value = line:match("(%S-)%s*:%s*(.*)")
    if param then
      request[param] = value
    end
  until line == ""
  -- Dcode les variables de formulaire passes via l'URL
  local filepath, vars = path:match("(.*)%?(.*)")
  if vars then
    path = filepath
    for k,v in vars:gmatch("([^&=]+)=([^&]+)") do
      request[urldecode(k)] = urldecode(v)
    end
  end
  -- Dtermine si l'URL est un fichier ou un rpertoire
  -- (contenant v. index.html) et agit en consquence.
  path = root .. urldecode(path)
  local lfs = require 'lfs'
  local data
  if not path:match("/$") then
    data = read_file(path, request)
  elseif lfs.attributes(path.."index.html") then
    data = read_file(path.."index.html", request)
  else
    data = read_directory(path)
  end
  send(client, ("HTTP/1.0 200 OK\r\nContent-Length: %d\r\n\z
    Server: Lua\r\n\r\n") : format(#data))
  send(client, data)
end

-- Dcode caractres spciaux dans l'URL (+ et %XX)
function urldecode(str)
  return str:gsub("%+", " "):gsub("%%(%w%w)", function(c)
    return string.char(tonumber(c, 16))
  end)
end

-- Lit un fichier et retourne son contenu dans une chane
function read_file(path, request)
  local f = assert(io.open(path, 'rb'))
  local content = f:read("*a")
  f:close()
  -- Cration de l'environnement pour les tags dynamiques
  local exports = {'assert', 'bit32', 'error', 'ipairs', 
    'math', 'next', 'pairs', 'select', 'string', 'table',
    'tonumber', 'tostring', 'type', 'unpack'}
  local env = {os={date=os.date, time=os.time, 
    difftime=os.difftime}, request=request}
  for _,n in ipairs(exports) do
    env[n] = _ENV[n]
  end
  function env.printf(...) 
    env.print(string.format(...)) 
  end
  -- Recherche les tags <?lua ?> et excute leur contenu
  content = content:gsub("()<%?[Ll][Uu][Aa](.-)%?>", 
    function(pos, chunk)
      local printed = ""
      env.print = function(...)
        for _,v in ipairs{...} do
          printed = printed .. tostring(v) .. " "
        end
      end
      local src = string.format("@%s[%d]", path, pos+5)
      local fct = load(chunk, src, "t", env) or 
        assert(load("return "..chunk, src, "t", env))
      local result = fct() or ""
      return printed .. result
    end)
  return content
end

-- Liste un rpertoire et retourne une page HTML avec
-- le nom des fichiers et leur taille.
function read_directory(path)
  local lfs = require 'lfs'
  local t = {
    "<html><head><title>Dossier ", path,
    "</title></head><body><h1>Dossier ", path,
    "</h1><table border=1><tr><th>Fichier</th>",
    "<th>Taille</th><th>Date</th></tr>"
  }
  for file in lfs.dir(path) do
    if file:sub(1,1) ~= '.' then
      local attr = assert(lfs.attributes(path..file))
      if attr.mode == "directory" then
        file = file .. "/"
      end
      local cols = {
        ('<a href="%s">%s</a>'):format(file, file),
        attr.size, 
        os.date(nil, attr.modification) }
      t[#t+1] = "<tr><td>"..table.concat(cols, "</td><td>")
        .."</td></tr>"
    end
  end
  t[#t+1] = "</table></body></html>"
  return table.concat(t)
end

-- Lance le serveur avec les arguments passs au script
main_server(...)
