#include <stdio.h>

extern "C" 
{
	#include <lua.h>						// inclut l'entte de l'API C de Lua
	#include <lauxlib.h>					// inclut l'entte auxiliaire de l'API C de Lua
	#include <lualib.h>						// inclut l'entte des librairies de l'API C de Lua
}

void stackdump_g(lua_State* l)
{
	int i;
	int top = lua_gettop(l);

	printf("total in stack %d\n",top);

	for (i = 1; i <= top; i++)
	{ /* repeat for each level */
		int t = lua_type(l, i);
		switch (t) {
			case LUA_TSTRING: /* strings */
				printf("string: '%s'\n", lua_tostring(l, i));
				break;
			case LUA_TBOOLEAN: /* booleans */
				printf("boolean %s\n",lua_toboolean(l, i) ? "true" : "false");
				break;
			case LUA_TNUMBER: /* numbers */
				printf("number: %g\n", lua_tonumber(l, i));
				break;
			default: /* other values */
				printf("%s\n", lua_typename(l, t));
				break;
		}
		printf("   "); /* put a separator */
	}
	printf("\n"); /* end the listing */
}

class AAA
{
private:
  // deux attributs entiers
  int m_a;
  int m_b;

public:
  // constructeur par dfaut  
  AAA()
  {
    m_a = 0;
    m_b = 0;
  }

  // fonction qui incrmente les attributs
  void Add(int _a, int _b)
  {
    m_a += _a;
    m_b += _b;
  }

  // fonction qui retourne la somme actuelle des deux attributs
  int Sum()
  {
    return m_a + m_b;
  }

  // enregistrement de la classe AAA dans un tat Lua donn 
  static void Register(lua_State* L)
  {
    // cration d'une mtatable nomme luaAAA
    luaL_newmetatable(L, "luaAAA");

    // ajoute les fonctions Create, Add et Sum  la mtatable
    lua_pushcfunction(L, AAA::Create);
    lua_setfield(L, -2, "Create");

    lua_pushcfunction(L, AAA::Add);
    lua_setfield(L, -2, "Add");

    lua_pushcfunction(L, AAA::Sum);
    lua_setfield(L, -2, "Sum");

    // fait pointer __index sur la mtatable elle-mme
    lua_pushvalue(L, -1);
    lua_setfield(L, -1, "__index");

    // cre une instance globale de la classe pour pouvoir
    // appeler les fonctions statiques
    lua_setglobal(L, "AAA");

    lua_pop(L, -1);
  }

  // cration d'une instance de la classe AAA   
  static int Create(lua_State* L)
  {
    // cration d'une nouvelle userdata  
    AAA** current = (AAA**)lua_newuserdata(L, sizeof(AAA*));

    // ajout de la nouvelle instance de la classe  
    *current = new AAA();

    // rcupration de la mtatable luaAAA
    luaL_getmetatable(L, "luaAAA");

    // assignation de la mtatable  la nouvelle instance
    lua_setmetatable(L, -2);

    return 1;
  }

  // rcupre l'instance courante de la classe AAA  
  static AAA* GetInstance(lua_State* L, int index)
  {
    return *((AAA**)luaL_checkudata(L, index, "luaAAA"));
  }

  // fonction Add qui sera appele en Lua
  static int Add(lua_State* L)
  {
    // rcupration de l'instance courante
    AAA* current = GetInstance(L, 1);

    // rcupration des paramtres
    int a, b;
    a = lua_tointeger(L, 2);
    b = lua_tointeger(L, 3);

    // appel de la mthode Add de l'instance
    current->Add(a, b);

    return 0;
  }

  // fonction Sum qui sera appele en Lua
  static int Sum(lua_State* L)
  {
    // rcupration de l'instance courante
    AAA* current = GetInstance(L, 1);

    // appel de la mthode Sum de l'instance
    int result = current->Sum();

    // ajout du rsultat dans la pile d'appels
    lua_pushnumber(L, result);

    return 1;
  }
};

int main ( int argc, char *argv[] )
{
	lua_State* L = luaL_newstate();		// cre un nouvel tat Lua
	luaL_openlibs(L);					// ouvre les librairies standards
	
	AAA::Register(L);
	
	if( luaL_dofile(L, "lua_cpp.lua") != LUA_OK )
	{
		printf("Erreur lors du chargement du fichier %s : %s\n",
      "lua_cpp.lua", lua_tostring(L, -1));
	}
	stackdump_g(L);
	lua_close(L);						// ferme et dtruit l'tat Lua

	return 0;
}
