#include "zlib.h"
#include "lua.h"
#include "lauxlib.h"

static int cksum_common(lua_State* L, int defval,
    uLong (*cksum)(uLong, const Bytef*, uInt)) {
  size_t srclen;
  const Bytef* src=(const Bytef*)luaL_checklstring(L,1,&srclen);
  uLong val = luaL_optunsigned(L, 2, defval);
  val = cksum(val, src, srclen);
  lua_pushunsigned(L, val);
  return 1;
}

static int zlib_adler(lua_State* L) {
  return cksum_common(L, 1, adler32);
}

static int zlib_crc(lua_State* L) {
  return cksum_common(L, 0, crc32);
}

static const luaL_Reg zlib_fcts[] = {
  { "crc",        zlib_crc },
  { "adler",      zlib_adler },
  { NULL, NULL }
};

int luaopen_zlib(lua_State* L) {
  luaL_newlib(L, zlib_fcts);
  lua_pushstring(L, ZLIB_VERSION);
  lua_setfield(L, -2, "_VERSION");
  return 1;
}
