#include "zlib.h"
#include "lua.h"
#include "lauxlib.h"

static int cksum_common(lua_State* L, int defval,
    uLong (*cksum)(uLong, const Bytef*, uInt)) {
  size_t srclen;
  const Bytef* src=(const Bytef*)luaL_checklstring(L,1,&srclen);
  uLong val = luaL_optunsigned(L, 2, defval);
  val = cksum(val, src, srclen);
  lua_pushunsigned(L, val);
  return 1;
}

static int zlib_adler(lua_State* L) {
  return cksum_common(L, 1, adler32);
}

static int zlib_crc(lua_State* L) {
  return cksum_common(L, 0, crc32);
}

static int compres_common(lua_State* L, int fdeflate) {
  size_t srclen;
  const Bytef* src=(const Bytef*)luaL_checklstring(L,1,&srclen);
  int level = luaL_optint(L, 2, Z_DEFAULT_COMPRESSION);
  uLong dstlen = srclen;
  Byte* dst;
  int res;
  while (1)
  {
    dstlen *= 2;
    lua_settop(L, 1);
    dst = (Bytef*)lua_newuserdata(L, dstlen);
    if (fdeflate)
      res = compress2(dst, &dstlen, src, srclen, level);
    else
      res = uncompress(dst, &dstlen, src, srclen);
    switch (res)
    {
    case Z_OK:
      lua_pushlstring(L, (const char*)dst, dstlen);
      return 1;
    case Z_BUF_ERROR:
      continue;
    case Z_MEM_ERROR:
      luaL_error(L, "Erreur mmoire durant la %scompression", 
        fdeflate ? "" : "d");
    case Z_DATA_ERROR:
      luaL_error(L, "Donnes compresses corrompues");
    case Z_STREAM_ERROR:
      luaL_error(L, "Niveau de compression incorrect");
    default:
      luaL_error(L, "Erreur zlib n %d", res);
    }
  }
}

static int zlib_compress(lua_State* L) {
  return compres_common(L, 1);
}

static int zlib_uncompress(lua_State* L) {
  return compres_common(L, 0);
}

static const luaL_Reg zlib_fcts[] = {
  { "crc",        zlib_crc },
  { "adler",      zlib_adler },
  { "compress",   zlib_compress },
  { "uncompress", zlib_uncompress },
  { NULL, NULL },
};

int luaopen_zlib(lua_State* L) {
  luaL_newlib(L, zlib_fcts);
  lua_pushstring(L, ZLIB_VERSION);
  lua_setfield(L, -2, "_VERSION");
  return 1;
}
